
import pbox2d.*;

import org.jbox2d.collision.shapes.*;
import org.jbox2d.common.*;
import org.jbox2d.dynamics.*;
import org.jbox2d.collision.*;
import org.jbox2d.dynamics.joints.*;
import java.util.ListIterator;
import java.util.LinkedList;

// A reference to our box2d world
PBox2D box2d;


// A list we'll use to track fixed objects
ArrayList boundaries;
ArrayList buttons;
ArrayList things;
ColorPicker colorPicker;


Tool currentTool;




void setup() {
  size(1024, 768);
  smooth();

  // Initialize box2d physics and create the world
  box2d = new PBox2D(this);
  box2d.createWorld();

  // Add some boundaries
  boundaries = new ArrayList();
  boundaries.add(new Boundary(100, 0, 924, 10));
  boundaries.add(new Boundary(100, 668, 924, 10));
  boundaries.add(new Boundary(100, 10, 10, 658));
  boundaries.add(new Boundary(1014, 10, 10, 658));

  // Add the tools
  RectangleTool rectangleTool = new RectangleTool(); // the rectangle tool creates rectangles
  CircleTool circleTool = new CircleTool();
  PencilTool pencilTool = new PencilTool();

  buttons = new ArrayList();
  buttons.add(new Button(0, 0, 100, 100, loadImage("rect-icon.png"), rectangleTool)); // this button activates the rectangle tool
  buttons.add(new Button(0, 100, 100, 100, loadImage("circle-icon.png"), circleTool));
  buttons.add(new Button(0, 200, 100, 100, loadImage("pencil-icon.png"), pencilTool));
  
  // Select the first button by defauls
  ((Button)buttons.get(0)).activate();

  colorPicker = new ColorPicker(0, 300, 100, 200);
  
  things = new ArrayList();
  

}

void draw() {
  background(255);

  // We must always step through time!
  box2d.step();

  // Show the boundaries
  for (int i = 0; i < boundaries.size(); i++) {
    Boundary boundary = (Boundary) boundaries.get(i);
    boundary.display();
  }

  // Show the interface
  for (int i = 0; i < buttons.size(); i++) {
    Button button = (Button) buttons.get(i);
    button.display();
  }

  colorPicker.display();

  
  // Show the stuff on the stage
  for (int i = 0; i < things.size(); i++) {
    Thing thing = (Thing) things.get(i);
    thing.display();
  }  

  currentTool.display();

  // 
  // // Here we create a dynamic gravity vector based on the location of our mouse
  // PVector g = new PVector(mouseX-width/2,mouseY-height/2);
  // g.normalize();
  // g.mult(10);
  // box2d.setGravity(g.x, -g.y);

}

void mousePressed() {
  // use the tool...
  if((mouseX > 110 && mouseX < 1024) && (mouseY > 10 && mouseY < 668)) {
    println("down on stage");
    currentTool.onStart();
  }
}

void mouseDragged() {
  if(currentTool.drawing) {
    currentTool.onDrag();
  }
}



void mouseReleased() {
  if((mouseX > 110 && mouseX < 1024) && (mouseY > 10 && mouseY < 668)) {
    println("up on stage");
    currentTool.onFinish();
  }
  else {
    currentTool.onCancel();
  }
}

